(*====================================================================
    File psguard.dpr

    Process Guardian.
    Project file.

    Copyright  2005-2009, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
======================================================================
$Id: psguard.dpr,v 1.2 2009/01/29 09:23:05 ariafan Exp $
======================================================================*)
program psguard;

uses
  Windows, SysUtils, PsApi;

{$R *.res}

const
  cSettings = 'Settings';
  cSleepInterval = 'SleepInterval';
  cApplicationName = 'ApplicationName';
  cParameters = 'Parameters';
  AppDelim: TSysCharSet = ['-', '/'];

resourcestring
  rsNoFileName = 'A module name is not defined.';
  rsDoublicateFileName = 'Watching on the module %s is already started on this computer.';
  rsErrorMsg = 'Error %s: %s';
  rsStartMsg = 'Start';
  rsStopMsg = 'Stop';
  rsReStartMsg = 'Restart';

var
  ProcessName, LogFileName, FullProgramName, Parameters: string;
  FirstRun: Boolean;
  NeedClose: Boolean;
  exename: string;

procedure WriteLog(Msg: string);
var
  F: TextFile;
begin
  AssignFile(F, LogFileName);
  try
    if FileExists(LogFileName) then Append(F) else ReWrite(F);
    Write(F, DateTimeToStr(Now) + ' ' + Msg + sLineBreak);
    Flush(F);
  finally
    CloseFile(F);
  end;
end;

function ReadParams(const FFileName, Section, Ident, Default: string): string;
const
  BufSize = 16384;
var
  Buffer{, P}: PChar;
  {EqualPos: Integer;}
begin
  GetMem(Buffer, BufSize);
  try
    if GetPrivateProfilestring(PChar(Section), PChar(Ident), PChar(Default), Buffer, BufSize,
      PChar(FFileName)) <> 0
    then Result := Buffer;
  finally
    FreeMem(Buffer, BufSize);
  end;
end;

function SearchProcess: THandle;
var
  hProc, hMod: THandle;
  aProcesses: array[0..1023] of DWORD;
  szProcessName: array[0..256] of Char;
  cbNeeded, cProcesses: DWORD;
  i: Integer;
  SuccSearch: Boolean;
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  buffName, S: string;

  function CheckSuccess(FSuccess: BOOL): BOOL;
  var
    FLastError: Integer;
    FLastErrorMsg: string;
  begin
    Result := FSuccess;
    if not Result then begin
      {   }
      FLastError := GetLastError;
      {   }
      FLastErrorMsg := Format(rsErrorMsg, [IntToStr(FLastError), SysErrorMessage(FLastError)]);
      {      }
      WriteLog(FLastErrorMsg);
    end;
  end;

begin
  Result := 0;
  SuccSearch := False;
  if EnumProcesses(@aProcesses, SizeOf(aProcesses), cbNeeded) then
  begin
    cProcesses := cbNeeded div sizeof(DWORD);
    for i := 0 to cProcesses - 1 do begin
      hProc := OpenProcess(PROCESS_VM_READ or PROCESS_QUERY_INFORMATION or SYNCHRONIZE, False, aProcesses[i]);
      if hProc <> 0 then
      try
        SetLastError(0);
        if EnumProcessModules(hProc, @hMod, SizeOf(hMod), cbNeeded)
        then begin
          GetModuleBaseName(hProc, hMod, szProcessName, SizeOf(szProcessName));
          buffName := PChar(@szProcessName[0]);
          if AnsiSameText(buffName, ProcessName) then
          begin
            Result := hProc;
            SuccSearch := True;
            Break;
          end;
        end;
      finally
        if not SuccSearch then
          CloseHandle(hProc);
      end
    end;
    if not SuccSearch then begin
      FillChar(StartupInfo, SizeOf(StartupInfo), 0);
      with StartupInfo do begin
        cb := SizeOf(StartupInfo);
        wShowWindow := SW_SHOWNORMAL;
      end;
      if CheckSuccess(CreateProcess(nil, PChar(FullProgramName + ' ' + Parameters), nil, nil, False,
        0, nil, nil, StartupInfo, ProcessInfo))
      then begin
        {      }
        if FirstRun then S := rsStartMsg
        else S := rsReStartMsg;
        WriteLog(S + ' ' + FullProgramName);
        FirstRun := False;
        Result := ProcessInfo.hProcess;
      end;
    end;
  end
end;

function GetCommand(const aCommandStr: string): string;
begin
  if (Length(aCommandStr) = 2) and (aCommandStr[1] in AppDelim)
    then Result := aCommandStr[2]
  else Result := EmptyStr;
end;

procedure MakeFullPath(var aPath: string);
begin
  if ExtractFilePath(aPath) = EmptyStr then
    aPath := IncludeTrailingPathDelimiter(GetCurrentDir()) + aPath;
end;

type
  THandlePair = packed record hEvent, hProcess: THandle end;

var
  SleepInterval: DWORD;
  HandlePair: THandlePair;
  SetupFileName: string;
  c, i: Integer;
  SelfName, s: string;

begin
  {  }
  FirstRun := True;
  NeedClose := False;

  {   }
  SelfName := ParamStr(0);
  exename := ChangeFileExt(ExtractFileName(SelfName), EmptyStr);
  {  }
  LogFileName := ChangeFileExt(SelfName, '.log');
  {   }
  SetupFileName := ChangeFileExt(SelfName, '.cfg');

  {   }
  i := 1;
  c := ParamCount;
  while i <= c do begin
    s := GetCommand(ParamStr(i));
    if 'u' = s then
      NeedClose := True
    else
    if AnsiSameText('l', s) and (i < c) then begin
      i := i + 1;
      LogFileName := ParamStr(i);
    end
    else
    if AnsiSameText('i', s) and (i < c) then begin
      i := i + 1;
      SetupFileName := ParamStr(i);
    end;
    i := i + 1;
  end;

  MakeFullPath(LogFileName);
  MakeFullPath(SetupFileName);

  {   -   }
  FullProgramName := ReadParams(SetupFileName, cSettings, cApplicationName, EmptyStr);
  {   }
  ProcessName := AnsiUpperCase(ExtractFileName(FullProgramName));
  if ProcessName = EmptyStr then
  begin
    MessageBox(0, PChar(rsNoFileName), PChar(exename), MB_ICONERROR + MB_OK);
    Exit;
  end;
  {  event }
  HandlePair.hEvent := CreateEvent(nil, False, False, PChar(exename + '_' + ProcessName));
  if (HandlePair.hEvent = 0) then begin
    WriteLog(Format(rsErrorMsg, ['CreateEvent', SysErrorMessage(GetLastError)]));
    Exit;
  end;
  if (GetLastError = ERROR_ALREADY_EXISTS) then begin
    {   }
    if NeedClose then
      SetEvent(HandlePair.hEvent)
    else
      WriteLog(Format(rsDoublicateFileName, [ProcessName]));
    Exit;
  end
  else begin
    if NeedClose then Exit;
  end;

  {  }
  Parameters := ReadParams(SetupFileName, cSettings, cParameters, EmptyStr);
  SleepInterval := DWORD(StrToIntDef(ReadParams(SetupFileName, cSettings, cSleepInterval, EmptyStr), 10000));

  {   }
  WriteLog(rsStartMsg + ' ' + exename);
  repeat
    HandlePair.hProcess := SearchProcess();
    if HandlePair.hProcess <> 0 then begin
      case WaitForMultipleObjects(2, @HandlePair, False, INFINITE) of
        WAIT_OBJECT_0: begin
          WriteLog(rsStopMsg + ' ' + exename);
          Break;
        end;
        WAIT_FAILED: begin
          WriteLog('WaitForMultipleObjects: ' + SysErrorMessage(GetLastError));
          Break;
        end;
      end;
      {  }
      if SleepInterval > 0 then
        Sleep(SleepInterval);
    end;
  until HandlePair.hProcess = 0;

end.

